
/*	A "C" Code Implementation of Microstepping for the 80C196KD  */

#pragma db          	/* compiles with debug */
#pragma code		/* produces pseudo ASM code in list */
#pragma model (kc)	/* compiles forthe KC */
#include <80C196.h>	/* includes the general 196 header */

static char reserve[9];			/* these two lines are required */
#pragma locate (reserve = 0x30)		/* for the Eval Board */

char dir;				/* global direction */
unsigned int dist,speed;		/* global distance and speed */

register unsigned int output;	/* controls PWM(1&2) duty cycles */
pragma locate(output=0x16)	/* puts "output" over PWM_control (1&2) */

register int index;		/* POINTS TO THE CURRENT TABLE LOCATION */
REGISTER LONG INT POSITION;	/* tracks the current motor position */

/*

This is the table of duty cycles that produce sine/cosine values on the
PWM2/PWM1 channels. When a value fromthis table is copies to "output" the
high order byte is in PWM2_control and the low order byte is in PWM_1con-
trol.  Each entry in the table corresponds to  PWM2=Sin(*) PWM1=cos(*).
Where * is 0,2.5,5,7.5,...355,357.5 (degrees).

*/

const int movement_table[144]=
{
0x80FF,0x86FF,0x8BFF,0x91FF,0x96FE,0x9CFD,0xA1FC,0xA6FA,
0xACF8,0xB1f6,0xB6F4,0xBBF2,0xCOEF,0xC5EC,0xC9E9,0xCEE6,
0xD2E2,0xD6DE,0xDBDB,0xDED6,0xE2D2,0xE6CE,0xE9C9,0xECC5,
0xEFC0,0xF2BB,0xF4B6,0xF6B1,0xF8AC,0xFAA6,0xFCA1,0xFD9C,
0xFE96,0xFF91,0xFF86,0xFF80,0xFF80,0xFF7A,0xFF75,0xFF6F,
0xFE6A,0xFD64,0xFC5F,0xFA5A,0xF854,0xF64F,0xF44A,0xF245,
0xEF40,0xEC3B,0xE937,0xE632,0xE22E,0xDE2A,0xDB25,0xD622,
0xD21E,0xCE1A,0xC917,0xC514,0xC011,0xBB0E,0xB60C,0xB10A,
0xAC08,0xA606,0xA104,0x9C03,0x9602,0x9101,0x8B00,0x8600,
0x5408,0x4F0A,0x4A0C,0x450E,0x4011,0x3B14,0x3717,0x321A,
0x2E1E,0x2A22,0x2525,0x222A,0x1E2E,0x1A32,0x1737,0x143B,
0x1140,0x0E45,0x0C4A,0x0A4F,0x0854,0x065A,0x045F,0x0364,
0x026A,0x016F,0x0075,0x007A,0x0080,0x0086,0x008B,0x0191,
0x0296,0x039C,0x04A1,0x06A6,0x08AC,0x0AB1,0x0CB6,0x0EBB,
0x11C0,0x14C5,0x17C9,0x1ACE,0x1ED2,0x22D6,0x25DB,0x2ADE,
0x2EE2,0x32E6,0x37E9,0x3BEC,0x4DEF,0x45F2,0x4AF4,0x4FF6,
0x54F8,0x5AFA,0x5FFC,0x64FD,0x6AFE,0xFFF,0x75FF,0x7AFF
};

/*

This is the procedure that moves the motor. Three parameters are passed to
it, direction, distance and speed.  dir=1 or 0, 0<=dist<=FFFFh,
0<=speed<=FFFFh.

*/

void move_it(unsigned char dir,unsigned int dist,unsigned int speed)
     {
     unsigned int temp,templ;
     for(temp=0;temp<dist;++temp)	/* loops # of microsteps */
	  {
	  if(dir)			/* chooses forward movement */
	       {
	       if(++index>143) index=0;	/* moves index forward 1 step */
		++position;		/* adds 1 to position */
	       }
  	  else				/* choose reverse movement */
	  {
	       if(-index<0) index=143;	/* moves index back 1 step */
	       -position;		/* subtracts 1 from position */
	       }
	  output=movement_table[index];	/* changes the duty cycles */
	  for(temp1=0;templ<speed;++temp1);	/* speed control */
}

main()
{
wsr=0;					/* select HWindow 0 */
ioc2=0;					/* ioc2.2=0 selects fast PWM mode */
wsr=1;					/* select HWindow 1 */
index=0;				/* point index to the movement_table[0] */
position=0;				/* initialize position */
output=movement_table[index];		/* load the PWM control registers */
t2control=0x0c;				/* now called ioc3, enables PWM1 and PWM2 */

dir=1;					/* selects forward direction */
dist=0;					/* motor is still until dist is changed */
speed=100;				/* selects a moderate speed */

/*

The while loop below runs continuously.  On the Eval Board the variables
dir, dist, and speed can be changed through Intel's Embedded Control
Monitor (ECM) while the KD is running.  When dist is changed to a non zero
value, the three parameters are passed to move_it and the motor moves dist
steps in the direction specified by dir.  When move_it is done dist is set
to zero and the motor remains stationary untill dist is changed through
ECM.

*/

while(1)
     {
     while(dist==0);			/* waits for a non zero dist */
     move_it(dir,dist,speed);		/* calls move_it */
     dist=0;				/* sets dist to 0 */
     }
}